#!/usr/local/bin/perl
#-------------------------------------------------------------------
# Change the location of perl on your server on the line above
#
# $cgi_dir  - url path to your cgi directory
# $my_url   - url path to this module
# $my_root  - path to the root web directory relative to the server root 
# require   - path to the library file relative to the server root 
#
# No other settings need to be changed.
#-------------------------------------------------------------------

$cgi_dir = "http://www.yourserver.com/cgi-bin";
$my_url = "http://www.yourserver.com/cgi-bin/lmadm.pl";
$my_root = "/home/myname/public_html";
require "/home/myname/cgi-bin/lmlib.pl";

#############################################################################
# ListMail ADMINSTRATION MODULE
#
# This script handle user and security maintainance of the ListMail system.
# Maintainance of these settings include
#
#     a) Maintenance of the admin password
#     b) Removal of subscribed users
#     c) Adding of unsubscribed non-member users
#     d) Exclusion of -
#          i)   IP Addresses
#          ii)  URL Addresses
#          iii) Email Addresses
#     e) Links to configuration and ListMail scripts
#
# ListMail Setup:
# ---------------
# 1) Modify only the first line and path locations above in all files.
# 2) Place all script files into the cgi-bin directory.
# 3) Create a subdirectory for ListMail.
# 4) Create two subdirectories in the newly created ListMail directory.
#    Both directories must be world read and writable.
#       a) Files - Storage of data files used by the system.
#       b) Upload - temporary storage for uploaded files
# 5) Run this script to create and customize the ListMail for your needs.
#
# NOTE: All files for data and upload may be stored within the same
# subdirectory.  Keep in mind that all files within the directory can
# be accessed by users.  You may want to add a file named 'index.htm'
# to prevent users the ability to browse the contents of the directory.
#
# This script will allow for uploading of files from a computer to a server
# as well as handling subscriptions for the list. To use this you must be
# using a browser which supports RFC1867 protocol.  You may wish to visit 
# http://www.cis.ohio-state.edu/htbin/rfc/rfc1867.html to learn more about it.
#############################################################################
# COPYRIGHT NOTICE          Copyright 1998 P.M.Systems  All Rights Reserved.
#
# ListMail may be used and modified free of charge by anyone so long as
# this copyright notice and the comments above remain intact.  By using this
# code you agree to indemnify Paul Mele and P.M.Systems from any liability
# that might arise from it's use.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  In other words, please ask first before you try and
# make money off of my program.
#
# Obtain permission before redistributing this software over the Internet or
# in any other medium.  In all cases copyright and header must remain intact.
#
# Archived at: http://www.pmsys.com/software/    contact: webmaster@pmsys.com
##############################################################################
# HISTORY
#
# 1.00 - Initial Release
# 1.10 - Added additional variables to be setup prior to executing  [10/15/98]
##############################################################################


#######################################################
# Initialize vars
#######################################################
$VN = "1.10";
$| = 1;
$date = localtime(time);

#######################################################
# Parse incoming data
#######################################################
$argcnt = &ParseInput;

#######################################################
# Get list configuration data
#######################################################
if ( $GLOBAL{'cfg'} ) {
    $cfgdata = $GLOBAL{'cfg'};
    $argcnt--;
}
else {
    $cfgdata = "/lmcfg.dat";
}
open (TEST, "$my_root$cfgdata") || die "Can't open config file $my_root$cfgdata.";
close (TEST);
require "$my_root$cfgdata";
if ($DATA_ROOT ne '' ) { $DATA_ROOT = $my_root.$DATA_ROOT; }

#######################################################
# security checks
#######################################################
if (@AUTHORIZED_URLS) { &CheckReferer; }
&CheckIp;
&CheckUrl;

#######################################################
# Check for exit (currently not used)
#######################################################
if( $GLOBAL{'exit'} ) { print "Location: $RETURN_URL\n\n"; }

#######################################################
# Check for previous admin usage
#######################################################
if( $GLOBAL{'admin'} ) {

    if ( (!$GLOBAL{'newpass'} || !$GLOBAL{'verifypass'}) ||
         ( $GLOBAL{'newpass'} ne  $GLOBAL{'verifypass'}) ) {
        &PassError;
    }
    &MakePass( $GLOBAL{'email'}, $GLOBAL{'newpass'} );
}
else {
    open(FILE, "$DATA_ROOT/$ADMIN_FILE") || &ModifyForm(1);
    close(FILE);
}

#######################################################
# Nothing parsed means we are at the login menu
#######################################################
if ( !$argcnt || $GLOBAL{'login'} ) { &Login; }

#######################################################
# Verify we have a valid email and password
#######################################################
&CheckParms($GLOBAL{'email'}, $GLOBAL{'password'});
$email = $GLOBAL{'email'};
$pass = $GLOBAL{'password'};
$value = $GLOBAL{'name'};
if ( $GLOBAL{'remove'} ) { $value = $GLOBAL{'lname'}; }
if( $GLOBAL{'encrypt'} ) { $encrypt=1; }else{ $encrypt=0; }
&VerifyUserPass(1);
&CheckEmail;

#######################################################
# Menus for verified admin
#######################################################
if( $GLOBAL{'menu'} ) { &Menu; }
if( $GLOBAL{'exclusion'} ) { &ExclusionMenu; }
if( $GLOBAL{'usermenu'} ) { &UserMenu; }

#######################################################
# List members, ip or url
#######################################################
if( $GLOBAL{'umain'} ) { &ShowList( 1 ); }
if( $GLOBAL{'newmain'} ) { &ShowList( 2 ); }
if( $GLOBAL{'ipmain'} ) { &ShowList( 3 ); }
if( $GLOBAL{'urlmain'} ) { &ShowList( 4 ); }
if( $GLOBAL{'emailmain'} ) { &ShowList( 5 ); }

#######################################################
# Show admin password form
#######################################################
if( $GLOBAL{'chgpwd'} ) { &ModifyForm; }

#######################################################
# Change admin password
#######################################################
if( $GLOBAL{'makepass'} ) {

    if ( (!$GLOBAL{'newpass'} || !$GLOBAL{'verifypass'}) ||
         ( $GLOBAL{'newpass'} ne  $GLOBAL{'verifypass'}) ||
         ( $GLOBAL{'newpass'} eq  $GLOBAL{'oldpass'}   ) ) {
        &PassError;
    }
    $pass = $GLOBAL{'oldpass'};
    $encrypt = 0;
    &VerifyUserPass(1);
    &MakePass( $email, $GLOBAL{'newpass'} );
}

#######################################################
# Users to add/remove
#######################################################
if( $GLOBAL{'user'} && $GLOBAL{'add'} ) { &ListAdd( 1 ); }
if( $GLOBAL{'user'} && $GLOBAL{'remove'} ) { &ListRemove( 1 ); }

#######################################################
# New email addresses to add/remove
#######################################################
if( $GLOBAL{'new'} && $GLOBAL{'add'} ) { &ListAdd( 2 ); }
if( $GLOBAL{'new'} && $GLOBAL{'remove'} ) { &ListRemove( 2 ); }

#######################################################
# IP Addresses to ignore
#######################################################
if( $GLOBAL{'ip'} && $GLOBAL{'add'} ) { &ListAdd( 3 ); }
if( $GLOBAL{'ip'} && $GLOBAL{'remove'} ) { &ListRemove( 3 ); }

#######################################################
# URL Addresses to ignore
#######################################################
if( $GLOBAL{'url'} && $GLOBAL{'add'} ) { &ListAdd( 4 ); }
if( $GLOBAL{'url'} && $GLOBAL{'remove'} ) { &ListRemove( 4 ); }

#######################################################
# Email Addresses to ignore
#######################################################
if( $GLOBAL{'addr'} && $GLOBAL{'add'} ) { &ListAdd( 5 ); }
if( $GLOBAL{'addr'} && $GLOBAL{'remove'} ) { &ListRemove( 5 ); }

#######################################################
# All done - we are outta here!
#######################################################
&Menu;
exit;


#######################################################
# Subroutine to display exclusion menu to user
#######################################################
sub ExclusionMenu {

    &Header("$LIST_NAME Exclusion Menu");
    print "<FORM METHOD=POST>\n";
    print "<p><font size='+1'><i>";
    print "Select from one of the following functions:";
    print "</i></font></p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "<table border='1' width='100%'>\n";
    print "<tr><td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=ipmain VALUE='IP Addresses'>\n";
    print "</td><td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=urlmain VALUE='URL Addresses'>\n";
    print "</td><td align='center'>\n";
    print "<INPUT TYPE=submit NAME=emailmain VALUE='Email Addresses'>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</td></tr>\n";
    print "<tr><td align='center'>\n";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Main Menu '>\n";
    &WriteUserInfo;
    print "</td></tr>\n";
    print "</table>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to add to a specified list.
#######################################################
sub ListAdd {

    my($type) = @_;
    my(@list, $file, $name, $dup, @write_fields, $rtn);

    # Lock file so if it is being used by more than one person at a time,
    # it shouldn't be destroyed.
    &CheckLock(5);

    # Read in file
    if ($type eq 1) {
        $file = "$DATA_ROOT/$USER_FILE";
        $rtn = "umain";
    }
    if ($type eq 2) {
        $file = "$DATA_ROOT/$ADDR_FILE";
        $rtn = "newmain";
    }
    if ($type eq 3) {
        $file = "$DATA_ROOT/$IP_FILE";
        $rtn = "ipmain";
    }
    if ($type eq 4) {
        $file = "$DATA_ROOT/$URL_FILE";
        $rtn = "urlmain";
    }
    if ($type eq 5) {
        $file = "$DATA_ROOT/$EMAIL_FILE";
        $rtn = "emailmain";
    }
    @list = &ReadFile( $file );

    # Check for duplicates
    $dup = 0;
    foreach $name (@list) {
        if ($type eq 1) { ($pwd, $name) = split(/\|/, $name); }
        if (lc($name) eq lc($value)) { $dup = 1; }
    }

    # Alert of duplicate entry
    if ( $dup ) {
        &Header("Duplicate Entry");
        print "<p>$value is already located in the file.</p>\n";
        print "<center><FORM METHOD=POST>";
        print "<INPUT TYPE=submit NAME=$rtn VALUE=' Ok '>\n";
        &WriteUserInfo;
        print "</center>\n"; 
        &Footer;
        &CleanUp;
    }

    # Not a duplicate entry, so re-open the file and write to it.
    else {
        open (LOG, ">>$file") || die "Can't open $file for output. [ListAdd]";
        if ($type eq 1) {
            @write_fields = (&CryptPass($pass), $email);
            print LOG join("\|", @write_fields) . "\n";
        }
        else {
            print LOG "$value\n";
        }
        close (LOG);
        &CleanUp;
        if ($type eq 2) {
            
            # Send complimentary subscription notice to user added by the Admin
            open(MAIL, "|$MAILPROG -t") || die "Can't open $MAILPROG!";
            print MAIL "TO: $value\n";
            print MAIL "FROM: $FROM\n";
            print MAIL "SUBJECT: $COMPLIMENT_SUBJ\n\n";
            if ($COMPLIMENT_FILE ne '') {
                open (COMPLIMENT, "$DATA_ROOT/$COMPLIMENT_FILE") || die "Can't open $COMPLIMENT_FILE file. [Admin User Add]";
                @list = <COMPLIMENT>;
                close (COMPLIMENT);
                print MAIL "@list\n";
            }
            else {
                print MAIL "This confirms your complimentary subscription to our mailing list for ";
                print MAIL "$LIST_NAME by the list administrator. If this subscription is ";
                print MAIL "unwanted, please unsubscribe as stated below.  To send to other list ";
                print MAIL "members you must subscribe by visiting http://$ENV{HTTP_HOST}$ENTRY_URL. ";
                print MAIL "or you may do nothing to begin and continue receiving postings from the list.\n";
            }
            print MAIL "\nIf you wish to unsubscribe at any time, please visit ";
            print MAIL "http://$ENV{HTTP_HOST}$ENTRY_URL, enter your eMail address ";
            print MAIL "leaving the password entry blank.\n\n";
            print MAIL "For questions or comments, please reply to $FROM.\n\n";
            close (MAIL);
        }
        &ShowList( $type );
    }
    exit;
}

#######################################################
# Subroutine to remove from a specified list.
#######################################################
sub ListRemove {

    my($type) = @_;
    my(@list, $name, $tmp, $found);
    my($match, $user_pass, $password, $user_name, $file, $rtn);


    # Lock file so if it is being used by more than one person
    # at a time, it shouldn't be destroyed.
    &CheckLock(5);

    # Read in file
    if ($type eq 1) {
        $file = "$DATA_ROOT/$USER_FILE";
        $rtn = "umain";
    }
    if ($type eq 2) {
        $file = "$DATA_ROOT/$ADDR_FILE";
        $rtn = "umain";
    }
    if ($type eq 3) {
        $file = "$DATA_ROOT/$IP_FILE";
        $rtn = "ipmain";
    }
    if ($type eq 4) {
        $file = "$DATA_ROOT/$URL_FILE";
        $rtn = "urlmain";
    }
    if ($type eq 5) {
        $file = "$DATA_ROOT/$EMAIL_FILE";
        $rtn = "emailmain";
    }
    @list = &ReadFile( $file );
    $match = 0;

    # Copy all the addresses except the one being deleted.
    open (LOG, ">$file") || die "Can't open $file for output. [ListRemove]";
    foreach $name (@list) {
        if ($type eq 1) {
            ($pwd, $tmp) = split(/\|/, $name);
        }
        else {
            $tmp = $name;
        }
        if (lc($tmp) eq lc($value) ) {
            $match = 1;
        }
        else {
            print LOG "$name\n";
        }
    }
    close (LOG);
    &CleanUp;  # Unlock the file.

    if ( !$match ) {

        &Header("Invalid Item");
        print "<p>The item you selected, $value, was not located in the file .</p>\n";
        print "<center><FORM METHOD=POST>";
        print "<INPUT TYPE=submit NAME=$rtn VALUE=' Ok '>\n";
        &WriteUserInfo;
        print "</center>\n"; 
        &Footer;
        exit;
    }
    &ShowList( $type );
}

#######################################################
# Subroutine to login user
#######################################################
sub Login {

    &Header("$LIST_NAME");
    print "<FORM METHOD='POST'>\n";
    print "<p>Welcome to the $LIST_NAME Admin Menu!  This menu is for administrators of $LIST_NAME only. ";
    print "You must enter both your username and password before selecting any available functions.</p>";
    print "<p><font size='+1'><i>Enter your Admin Name and Password:</i></font></p>\n";
    print "<table border='1' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "Admin Name: <INPUT TYPE=text NAME=email SIZE=30><BR>\n";
    print "</td><td>\n";
    print "Password: <INPUT TYPE=password NAME=password SIZE=10><BR>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td align='center' colspan='2'>\n";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Enter '>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Cancel</a> ]</p>\n";
    print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'></FORM>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</FORM>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to display verification of password change
#######################################################
sub MakePass {

    my($name, $newpass) = @_;
    my(@write_fields);

    # Write password to file
    $email = $name;
    $pass = &CryptPass($newpass);
    @write_fields = ($pass, $name);
    open (FILE, ">$DATA_ROOT/$ADMIN_FILE") || die "Can't open $ADMIN_FILE for output. [MakePass]";
    print FILE join("\|", @write_fields) . "\n";
    close (FILE);

    # Display html return page
    &Header("Successful Password Change");
    print "<p>Please record your user information below in order to regain ";
    print "access for admin functions.</p>\n";
    print "<p>Password: $newpass</p>";
    print "<center><FORM METHOD=POST>";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Menu '>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a> ]</p>\n";
    &WriteUserInfo;
    print "</center>\n"; 
    &Footer;
    exit;
}

#######################################################
# Subroutine to display main menu to user
#######################################################
sub Menu {

    &Header("$LIST_NAME Admin Menu");
    print "<FORM METHOD=POST>\n";
    print "<p><font size='+1'><i>";
    print "Select from one of the following functions:";
    print "</i></font></p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "<table border='1' width='100%'>\n";
    print "<tr><td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=chgpwd VALUE='Change Admin Pwd'>\n";
    print "</td><td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=exclusion VALUE='Exclusions'>\n";
    print "</td><td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=usermenu VALUE='User Maintenance'>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</td></tr>\n";
    print "<tr><td align='center'>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a> | \n";
    print "<a href='$cgi_dir/lmcfg.pl?email=$email&password=$pass&encrypt=1&cfg=$cfgdata&menu=1'>List Configuration</a> | \n";
    open (USER, "$DATA_ROOT/$USER_FILE");
    while (<USER>) {
        chop ($_);
        ($user_pass, $user_name) = split(/\|/, $_);
        if (lc($user_name) eq lc($email)) { last; }
    }
    close (USER);
    print "<a href='$cgi_dir/lmail.pl?email=$email&password=$user_pass&encrypt=1&cfg=$cfgdata&menu=1'>List Mail</a> ]</p>\n";
    &WriteUserInfo;
    print "</td></tr>\n";
    print "</table>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to modify admin password
#######################################################
sub ModifyForm {

    my($new) = @_;

    &Header("Admin Password Change");
    print "<FORM METHOD='POST'>\n";
    if ( $new ) {
        print "<p>Initiating Administrative Functions for first time usage -</p>\n";
        print "<table border='1' cellpadding='3' cellspacing='3' width='80%'>\n";
        print "<tr><td>\n";
        print "Admin Name:</td><td><INPUT TYPE=text NAME=email size=35>\n";
        print "</td></tr>\n";
    }
    else {
        print "<table border='1' cellpadding='3' cellspacing='3' width='80%'>\n";
        print "<tr><td>\n";
        print "<p>Current password:</p>\n";
        print "</td><td>\n";
        print "<INPUT TYPE=password NAME=oldpass size=35>";
        print "</td></tr>\n";
    }
    print "<tr><td>\n";
    print "<p>New Password:</p>\n";
    print "</td><td>\n";
    print "<INPUT TYPE=password NAME=newpass size=35>";
    print "</td></tr>\n";
    print "<tr><td>\n";
    print "<p>Verify Password:</p>\n";
    print "</td><td>\n";
    print "<INPUT TYPE=password NAME=verifypass size=35>";
    print "</td></tr>\n";
    print "</table>\n";
    print "<p> </p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td align='center'>\n";
    if ( $new ) {
        print "<INPUT TYPE=submit NAME=admin VALUE='Set Password'>\n";
        print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a> ]</p>\n";
        print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'>\n";
        print "</FORM>\n";
    }
    else {
        print "<INPUT TYPE=submit NAME=makepass VALUE=Change>\n";
        print "<INPUT TYPE=submit NAME=menu VALUE=Cancel>\n";
        &WriteUserInfo;
    }
    print "</td></tr>\n";
    print "</table>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to display user/pass do not match
#######################################################
sub PassError {

    &Header("Password Match Error");
    print "<p>The new password and verify password do not match or are blank.</p>\n";
    print "<center><FORM METHOD=POST>";
    print "<INPUT TYPE=submit NAME=menu VALUE=Return>\n";
    &WriteUserInfo;
    print "</center>\n"; 
    &Footer;
    exit;
}

#######################################################
# Subroutine to list specific items in a list
#######################################################
sub ShowList {

    my($type) = @_;
    my(@list, $pwd, $tmp, $line, $cnt);


    if ($type eq 1) {
        @list = &ReadFile( "$DATA_ROOT/$USER_FILE" );
        &Header("$LIST_NAME Members");
        print "<p>The following members are subscribed to the list:</p>\n";
    }
    if ($type eq 2) {
        @list = &ReadFile( "$DATA_ROOT/$ADDR_FILE" );
        &Header("$LIST_NAME Additional Email Addresses");
        print "<p>The following Email Addresses are not subscribed for access \n";
        print "but are sent mailings as part of the list:</p>\n";
    }
    if ($type eq 3) {
        @list = &ReadFile( "$DATA_ROOT/$IP_FILE" );
        &Header("$LIST_NAME Unauthorized IP Addresses");
        print "<p>The following IP Addresses are not authorized for access:</p>\n";
    }
    if ($type eq 4) {
        @list = &ReadFile( "$DATA_ROOT/$URL_FILE" );
        &Header("$LIST_NAME Unauthorized URLs");
        print "<p>The following URL Addresses are not authorized for access:</p>\n";
    }
    if ($type eq 5) {
        @list = &ReadFile( "$DATA_ROOT/$EMAIL_FILE" );
        &Header("$LIST_NAME Unauthorized Email Addresses");
        print "<p>The following Email Addresses are not authorized for access:</p>\n";
    }
    print "<FORM METHOD=POST>\n";
    print "<table border='1' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td align='center'>\n";
    if ( @list ) {
        print "<select name=lname>\n";
        foreach $line (@list) {
            if ( $type eq 1 ) {
                ($pwd, $tmp) = split(/\|/, $line);
                print "<option>$tmp</option>\n";
            }
            else {
                print "<option>$line</option>\n";
            }
        }
        print "</select>  \n";
        print "<INPUT TYPE=submit NAME=remove VALUE=Remove>\n";
    }
    else {
        if ( $type eq 1 ) { print "No Members Subscribed"; }
    }    
    if ($type eq 1) { $tmp = "user"; }
    if ($type eq 2) { $tmp = "new"; }
    if ($type eq 3) { $tmp = "ip"; }
    if ($type eq 4) { $tmp = "url"; }
    if ($type eq 5) { $tmp = "addr"; }
    if ($type ne 1) {
        if ( @list ) { print "</td><td align='center'>\n"; }
        print "<INPUT TYPE=text NAME=name SIZE=25> \n";
        print "<INPUT TYPE=submit NAME=add VALUE='Add New'>\n";
    }
    print "<INPUT TYPE=hidden NAME=$tmp VALUE=1>\n";
    print "</td></tr></table>\n";
    print "<p> </p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td align='center'>\n";
    if ($type gt 2) {
        print "<INPUT TYPE=submit NAME=exclusion VALUE=Return>\n";
    }
    else {
        print "<INPUT TYPE=submit NAME=usermenu VALUE=Return>\n";
    }
    &WriteUserInfo;
    print "</td></tr></table>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to display user menu to admin
#######################################################
sub UserMenu {

    &Header("$LIST_NAME User Menu");
    print "<FORM METHOD=POST>\n";
    print "<p><font size='+1'><i>";
    print "Select from one of the following functions:";
    print "</i></font></p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "<table border='1' width='100%'>\n";
    print "<tr><td align='center' width='50%'>\n";
    print "<INPUT TYPE=submit NAME=newmain VALUE='Non-Member Address List'>\n";
    print "</td><td align='center'>\n";
    print "<INPUT TYPE=submit NAME=umain VALUE='Subscribed Members'>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</td></tr>\n";
    print "<tr><td align='center'>\n";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Main Menu '>\n";
    &WriteUserInfo;
    print "</td></tr>\n";
    print "</table>\n";
    &Footer;
    exit;
}

# end of script 
