#!/usr/local/bin/perl
#-------------------------------------------------------------------
# Change the location of perl on your server on the line above
#
# $cgi_dir  - url path to your cgi directory
# $my_url   - url path to this module
# $my_root  - path to the root web directory relative to the server root 
# require   - path to the library file relative to the server root 
#
# No other settings need to be changed.
#-------------------------------------------------------------------

$cgi_dir = "http://www.yourserver.com/cgi-bin";
$my_url = "http://www.yourserver.com/cgi-bin/lmadm.pl";
$my_root = "/home/myname/public_html";
require "/home/myname/cgi-bin/lmlib.pl";

#############################################################################
# ListMail MAIN MODULE
#
# This script is what users of the ListMail system use to access the system.
# From this, users can subscribe, unsubscribe, send mail and list all members
# of the list.  When the adminstrator of the list logs on, additional links
# to configuration and ListMail scripts will be accessable.
#
# ListMail Setup:
# ---------------
# 1) Modify only the first line and path locations in all files.
# 2) Place all script files into the cgi-bin directory.
# 3) Create a subdirectory for ListMail.
# 4) Create two subdirectories in the newly created ListMail directory.
#    Both directories must be world read and writable.
#       a) Files - Storage of data files used by the system.
#       b) Upload - temporary storage for uploaded files
# 5) Run this script to create and customize the ListMail for your needs.
#
# NOTE: All files for data and upload may be stored within the same
# subdirectory.  Keep in mind that all files within the directory can
# be accessed by users.  You may want to add a file named 'index.htm'
# to prevent users the ability to browse the contents of the directory.
#
# This script will allow for uploading of files from a computer to a server
# as well as handling subscriptions for the list. To use this you must be
# using a browser which supports RFC1867 protocol.  You may wish to visit 
# http://www.cis.ohio-state.edu/htbin/rfc/rfc1867.html to learn more about it.
#############################################################################
# COPYRIGHT NOTICE          Copyright 1998 P.M.Systems  All Rights Reserved.
#
# ListMail may be used and modified free of charge by anyone so long as
# this copyright notice and the comments above remain intact.  By using this
# code you agree to indemnify Paul Mele and P.M.Systems from any liability
# that might arise from it's use.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  In other words, please ask first before you try and
# make money off of my program.
#
# Obtain permission before redistributing this software over the Internet or
# in any other medium.  In all cases copyright and header must remain intact.
#
# Archived at: http://www.pmsys.com/software/    contact: webmaster@pmsys.com
##############################################################################
# HISTORY
#
# 1.00 - Initial Release
# 1.10 - Added additional variables to be setup prior to executing  [10/15/98]
# 1.11 - Fixed verify routine to check for file before writing  [01/02/99]
# 1.12 - Correct subroutine call syntax error in Login [07/22/99]
##############################################################################

#######################################################
# Initialize vars
#######################################################
$VN = "1.12";
$| = 1;
$date = localtime(time);

#######################################################
# Parse incoming data
#######################################################
$argcnt = &ParseInput;

#######################################################
# Get list configuration data
#######################################################
if ( $GLOBAL{'cfg'} ) {
    $cfgdata = $GLOBAL{'cfg'};
    $argcnt--;
}
open (TEST, "$my_root$cfgdata") || die "Can't open config file $my_root$cfgdata.";
close (TEST);
require "$my_root$cfgdata";
if ($DATA_ROOT ne '' ) { $DATA_ROOT = $my_root.$DATA_ROOT; }

#######################################################
# security checks
#######################################################
if (@AUTHORIZED_URLS) { &CheckReferer; }
&CheckIp;
&CheckUrl;

#######################################################
# Nothing parsed means we are at the login menu
#######################################################
if ( !$argcnt || $GLOBAL{'login'} ) { &Login; }

#######################################################
# Check for exit (currently not used)
#######################################################
if( $GLOBAL{'exit'} ) { print "Location: $RETURN_URL\n\n"; }
    
#######################################################
# Allow front page unsubscribes w/o password entry
#######################################################
if( $GLOBAL{'remove'} ) {
    $email = $GLOBAL{'email'};
    $pass = $GLOBAL{'password'};
    $encrypt=0;
    &Unsubscribe;
}

#######################################################
# Verify we have a valid email and password
#######################################################
&CheckParms($GLOBAL{'email'}, $GLOBAL{'password'});
$email = $GLOBAL{'email'};
$pass = $GLOBAL{'password'};
if( $GLOBAL{'encrypt'} ) { $encrypt=1; }else{ $encrypt=0; }
&CheckEmail;

#######################################################
# Subscribe to list
#######################################################
if( $GLOBAL{'subscribe'} ) {
    if( !$GLOBAL{'verify'} ) { &Verify; }
    &Subscribe;
}

#######################################################
# Remainder of routines require verified users
#######################################################
&VerifyUserPass;

#######################################################
# Authorized user at menu
#######################################################
if( $GLOBAL{'menu'} ) { &Menu; }

#######################################################
# Unsubscribe from list
#######################################################
if( $GLOBAL{'unsubscribe'} ) { &Unsubscribe; }

#######################################################
# Display members of the list
#######################################################
if( $GLOBAL{'list'} ) { &ListUsers; }

#######################################################
# Compose message for list members
#######################################################
if( $GLOBAL{'upload'} ) { &UploadForm; }

#######################################################
# Upload data and text supplied in upload form to list
#######################################################
if( $GLOBAL{'send'} ) {
    &Send( $GLOBAL{'FILE_NAME'}, $GLOBAL{'mailself'},
           $GLOBAL{'subject'}, $GLOBAL{'comment'} );
}

#######################################################
# All done - we are outta here!
#######################################################
exit;



#######################################################
# Subroutine to list members of list
#######################################################
sub ListUsers {

    my (@addrlist, @userlist, @list, $line, $user, $domain, $pwd, $addr, $cnt);

    @userlist = &ReadFile( "$DATA_ROOT/$USER_FILE" );  # User list
    @addrlist = &ReadFile( "$DATA_ROOT/$ADDR_FILE" );  # Address List (Admin)

    # strip passwords off user list and add it to address list
    if ( @userlist ) {
        $cnt = @addrlist;
        foreach $line (@userlist) {
            ($pwd, @addrlist[$cnt]) = split(/\|/, $line);
            $cnt++;
        }
    }
    @list = sort(@addrlist);

    # Strip off domain name and display results mail names only
    &Header("$LIST_NAME Members");
    if ( @list ) {
        print "<p><ol>\n";
        foreach $line (@list) {
            ($user, $domain) = split(/@/, $line);
            print "<li>$user</li>\n";
        }
        print "</ol></p>\n";
    }
    else {
        print "<p><b>No Members</b></p>\n";
    }
    print "<center><FORM METHOD=POST>";
    print "<INPUT TYPE=submit NAME=menu VALUE=Return>\n";
    &WriteUserInfo;
    print "</center>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to login user
#######################################################
sub Login {

    my (@list, $line);

    &Header("$LIST_NAME");
    @list = &ReadFile( "$DATA_ROOT/$WELCOME_FILE" );
    if ( @list ) {
        foreach $line ( @list ) {
            print "$line\n";
        }
    }
    else {
        print "<p>Welcome to $LIST_NAME!  This list is for members only.  In order to subscribe, ";
        print "remove your name, or send messages and attachments to other members of the list ";
        print "you must enter both your email address and a password.  Once subscribed and ";
        print "confirmed as a user, select 'Send' to gain access to the message upload form.</p>";
    }
    print "<p><font size='+1'><i>Enter your eMail Address and Password:</i></font></p>\n";
    print "<FORM METHOD='POST'>\n";

    print "<table border='1' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "eMail Address: <INPUT TYPE=text NAME=email SIZE=30><BR>\n";
    print "</td><td>\n";
    print "Password: <INPUT TYPE=password NAME=password SIZE=10><BR>\n";
    print "</td></tr>\n";
    print "</table>\n";

    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "<table border='0' width='100%'><tr>\n";
    print "<td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Enter '></td>\n";
    print "<td align='center' width='33%'>\n";
    print "<INPUT TYPE=submit NAME=subscribe VALUE=Subscribe></td>\n";
    print "<td align='center'>\n";
    print "<INPUT TYPE=submit NAME=remove VALUE=Unsubscribe></td>\n";
    print "</tr>\n";
    print "</table>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a> ]</p>\n";
    print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'></FORM>\n";
    print "<p> </p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='100%'>\n";
    print "<tr><td>\n";
    print "<font size=2><b><i>NOTE:<b> If you are receiving mailings from this list, but have never subscribed or been assigned \n";
    print "a password then enter your email address leaving the password field blank to unsubscribe from the list.</i></font>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</FORM>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to send BCC copies of subs/unsubs
# to recipient defined in $FROM
#######################################################
sub LogUser {

    my($onoff) = @_;  # 0 - LogOff / 1 - LogOn

    # Log subscriptions and removals to log file
    if ( $LOG_SUBS ne '' ) {
        open (LOG, ">>$DATA_ROOT/$LOG_SUBS") || die "Can't open $LOG_SUBS for output [LogUser].";
        print LOG "Date = $date\n";
        print LOG "User = $email\n";
        if ( $onoff ne 0 ) {
            print LOG "Subscribed\n";
        }
        else {
            print LOG "Removed\n";
        }
        print LOG "=================\n";
        close (LOG);
    }

    # Send confirmation message to owner of list
    if ( $SEND_BCC ) {
        sleep(2);
        open(MAIL, "|$MAILPROG -t") || die "Can't open $MAILPROG! [LogUser]";
        print MAIL "TO: $FROM\n";
        if ( $onoff ) {
            print MAIL "SUBJECT: New Subscription for $LIST_NAME\n\n";
            print MAIL "A new user has subscribed to the list\n\n";
            print MAIL "Username: $email\n";
            print MAIL "Password: $pass\n\n";
        }
        else {
            print MAIL "SUBJECT: User unsubscribe for $LIST_NAME\n\n";
            print MAIL "The following user has unsubscribed from the list\n\n";
            print MAIL "Username: $email\n\n";
        }
        print MAIL "<p>This message is being sent as configured by the owner for $LIST_NAME. ";
        print MAIL "To stop receiving these confirmations, set \$SEND_BCC=0 in the configuration file.</p>\n\n";
        close (MAIL);
    }
}

#######################################################
# Subroutine to display main menu to user
#######################################################
sub Menu {

    my ($col_pct, $tmp, $admin, $name);

    # Check to see if list admin logged in
    $admin = 0;
    open (USER, "$DATA_ROOT/$ADMIN_FILE");
    while (<USER>) {
        chop ($_);
        ($tmp, $name) = split(/\|/, $_);
        if (lc($name) eq lc($email)) { $admin = 1; }
    }
    close( USER );
    $col_pct = "33%";

    # Display menu to user
    &Header("$LIST_NAME");
    print "<FORM METHOD='POST'>\n";
    print "<p><font size='+1'><i>";
    print "Select from one of the following functions:";
    print "</i></font></p>\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='80%'>\n";
    print "<tr><td>\n";
    print "<table border='1' width='100%'><tr>\n";
    print "<td align='center' width='$col_pct'>\n";
    print "<INPUT TYPE=submit NAME=upload VALUE='Send to List'>\n";
    print "</td><td align='center' width='$col_pct'>\n";
    print "<INPUT TYPE=submit NAME=list VALUE='List Members'>\n";
    print "</td><td align='center' width='$col_pct'>\n";
    print "<INPUT TYPE=submit NAME=unsubscribe VALUE=Unsubscribe>\n";
    print "</td></tr>\n";
    print "</table>\n";
    print "</td></tr><td align='center'>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a>";
    if ($admin ne 0) {

        # Admin links
        # Get password for administration.  Passwords for normal
        # list usage and admin are encoded differently
        open (USER, "$DATA_ROOT/$ADMIN_FILE") || &Menu;
        while (<USER>) {
            chop ($_);
            ($user_pass, $user_name) = split(/\|/, $_);
            if (lc($user_name) eq lc($email)) { last; }
        }
        close (USER);
        print " | <a href='$cgi_dir/lmadm.pl?email=$email&password=$user_pass&encrypt=1&cfg=$cfgdata&menu=1'>Adminstration</a> | \n";
        print "<a href='$cgi_dir/lmcfg.pl?email=$email&password=$user_pass&encrypt=1&cfg=$cfgdata&menu=1'>Configuration</a>\n";
    }
    print " ]</p>\n";
    print "</td></tr>\n";
    print "</table>\n";
    &WriteUserInfo;
    &Footer;
    exit;
}

#######################################################
# Subroutine to send message to list members
#######################################################
sub Send {

    my ($file_name, $cc, $subject, $comment) = @_;
    my ($remote_name, $remote_file, $file_stats, @status_list, $cnt);

    # Upload file if we have one
    if ( $file_name ne '' ) {

        # Extract just the name of the file w/o path
        $remote_name = $file_name;
        $remote_name =~ s/.+\\([^\\]+)$|.+\/([^\/]+)$/$1/;

        # Add our upload path to name of file
        $remote_file = $my_root.$UPLOAD_DIR.$remote_name;

        # Upload message/file from user
        &UploadFile($file_name,$remote_file);
        &Header("Message Upload and Send Results");
    }
    else { &Header("Message Send Results"); }

    # Send message to list
    print "<P>Sending message to list members...</P>\n";
    $cnt = &DoMailList($remote_name, $remote_file, $cc, $subject, $comment);

    # Display send results
    print "<P><PRE>";
    if ( $remote_file ne '' ) {
        $file_stats = `ls -la $remote_file`;
        @status_list = split(/\s+/,  $file_stats);
        print "Remote File Name : <FONT COLOR=\#FF0000>$file_name</FONT>\n";
        print "File Name : $remote_name\n";
        print "File Size : $status_list[4]\n";
        print "Local Time: $status_list[5] $status_list[6] $status_list[7]\n";
    }
    print "Subject: $SUBJ_PREFIX ";
    if ( $subject ne '' ) { print "$subject\n"; }else{ print "$remote_name\n"; }
    print "Recipients: $cnt\n";
    print "</PRE></P>";
    print "<p>Your message was successfully sent to all members of the list. ";
    print "Thank you for your submission!</p>";
    print "<FORM METHOD=POST>\n";
    print "<INPUT TYPE=submit NAME=upload VALUE='Send Another Message'>\n";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Menu '>\n";
    &WriteUserInfo;
    &Footer;
    exit;
}

#######################################################
# Subroutine to check to make sure eMail address being
# submitted isn't already on the recipient list and
# subscribe user to the list.
#######################################################
sub Subscribe {

    my(@line, $dup, $addr, $pwd);

    # Check security
    &CheckEmail;

    # Lock file so if it is being used by more than one
    # person at a time, it shouldn't be destroyed.
    &CheckLock(5);

    # Check for duplicates
    $dup = 0;
    @list = &ReadFile( "$DATA_ROOT/$USER_FILE" );
    foreach $line (@list) {

        # split password and user and compare
        ($pwd, $addr) = split(/\|/, $line);
        if (lc($addr) eq lc($email)) { $dup = 1; }
    }

    # Alert of duplicate entry, unlock the file and exit program
    if ( $dup ) {
        &Header("Duplicate Address");
        print "<p>$email is already on this mailing list.</p>\n";
        $email = '';
        $pass = '';
        print "<CENTER><FORM METHOD=POST>\n";
        print "<INPUT TYPE=submit NAME=login VALUE=Return>\n";
        print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'></FORM>\n";
        print "</CENTER>\n";
        &Footer;
    }

    # Not a duplicate entry, so re-open the file and write to it.
    else {

        # Write password to file
        $enc_pass = &CryptPass($pass);
        @write_fields = ($enc_pass, $email);
        open (LOG, ">>$DATA_ROOT/$USER_FILE") || die "Can't open $USER_FILE for append. [Subscribe]";
        print LOG join("\|", @write_fields) . "\n";
        close (LOG);

        # Check admin address file and remove if found
        @list = &ReadFile( "$DATA_ROOT/$ADDR_FILE" );
        open (LOG, ">$DATA_ROOT/$ADDR_FILE") || die "Can't open $ADDR_FILE for output. [Subscribe]";
        foreach $line (@list) {
            if (lc($line) ne lc($email)) { print LOG "$line\n"; }
        }
        close (LOG);

        # Send confirmation email to new subscriber
        open(MAIL, "|$MAILPROG -t") || die "Can't open $MAILPROG!";
        print MAIL "TO: $email\n";
        print MAIL "FROM: $FROM\n";
        print MAIL "SUBJECT: $CON_SUBJ\n\n";
        if ($CONFIRM_FILE ne '') {
            open (CONFIRM, "$DATA_ROOT/$CONFIRM_FILE") || die "Can't open CONFIRM_FILE file. [Subscribe]";
            @list = <CONFIRM>;
            close (CONFIRM);
            print MAIL "@list\n";
        }
        else {
            print MAIL "This confirms that you are now on the mailing list for $LIST_NAME.\n";
        }
        print MAIL "If you wish to unsubscribe at any time, please visit ";
        print MAIL "http://$ENV{HTTP_HOST}$ENTRY_URL and \n";
        print MAIL "enter your eMail address and password below.\n\n";
        print MAIL "Username: $email\n";
        print MAIL "Password: $pass\n";
        close (MAIL);

        # Send confirmation message to owner of list
        &LogUser(1);

        # Display html return page
        &Header("Successful Subcription");
        print "<h2>Thank you!</h2>\n";
        print "<p>Congratulations, $email has been added to the mailing list. ";
        print "You should receive confirmation eMail to that address shortly. ";
        print "Please record your user information below in order to gain access for mailing ";
        print "to list members or unsubscribing from the list.</p>\n";
        print "<p><PRE>";
        print "Username: $email<BR>";
        print "Password: $pass";
        print "</PRE></p>";
        print "<p>You may now use $LIST_NAME to send mailings.</p>";
        $pass = $enc_pass;
        print "<center>\n"; 
        print "<FORM METHOD=POST>\n";
        print "<INPUT TYPE=submit NAME=menu VALUE=' Menu '>\n";
        &WriteUserInfo;
        print "</center>\n"; 
        &Footer;
    }
    &CleanUp;
    exit;
}

#######################################################
# Subroutine to check to make sure eMail address being
# submitted is already on the recipient list and
# unsubscribe user from the list.
#######################################################
sub Unsubscribe {

    my (@list, $line, $addr, $pwd, $match, $found);

    # Lock file so if it is being used by more than
    # one person at a time, it shouldn't be destroyed.
    &CheckLock(5);

    # Check user file if we have both email and pass
    if ( $email ne '' && $pass ne '' ) {

        # Put addresses into an array
        @list = &ReadFile( "$DATA_ROOT/$USER_FILE" );

        # Reopen and copy all active records back
        $found = 0;
        open (LOG, ">$DATA_ROOT/$USER_FILE") || die "Can't open $USER_FILE for output [Unsubscribe]";
        foreach $line (@list) {
            $match = 0;

            # split password and user and compare
            ($pwd, $addr) = split(/\|/, $line);
            if (lc($addr) eq lc($email)) {
                if ( $encrypt ) {
                    if ($pass eq $pwd) { $match = 1; }
                }
                else {
                    $salt = substr ($pwd,0,2);
                    $tmp = crypt($pass,$salt);
                    if ($tmp eq $pwd) {
                        $pass = $tmp;
                        $match = 1;
                    }
                }
            }
            if ( !$match) { print LOG "$line\n"; }else{ $found = 1; }
        }
        close (LOG);
    }

    # Check admin address file and remove if not found in user file
    if ( !$found ) {

        # Check admin file if we an email address
        if ( $email ne '' ) {

            @list = &ReadFile( "$DATA_ROOT/$ADDR_FILE" );
            open (LOG, ">$DATA_ROOT/$ADDR_FILE") || die "Can't open $ADDR_FILE for output. [Subscribe]";
            foreach $line (@list) {
                if (lc($line) ne lc($email)) {
                    print LOG "$line\n";
                }
                else {
                    $found = 1;
                }
            }
            close (LOG);
        }
    }
        
    # Did we find anything matching
    if ( !$found ) {
        &Header("Invalid address");
        print "<p>The email address you entered, $email, is not authorized for the $LIST_NAME.</p>\n";
        print "<center>\n"; 
        print "<FORM METHOD=POST>";
        print "<INPUT TYPE=submit NAME=login VALUE=' Retry '>\n";
        &WriteUserInfo;
        print "</center>\n"; 
        &Footer;
    }
    else {

        # Alert user to completion of task.
        &Header("Unsubscribe Successful");
        print "<p>The user at the eMail address $email is no longer on this mailing list.<br>\n";
        print "Thank you for participating!</p>\n";
        print "<p align='center'>";
        print "<FORM METHOD=POST>";
    print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a> ]</p>\n";
        print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'></FORM>\n";
        print "</FORM></p>";
        &Footer;

        # Send confirmation message to owner of list
        &LogUser(0);
    }

    # Unlock the file.
    &CleanUp;
    exit;
}

#######################################################
# Subroutine to display upload form to user
#######################################################
sub UploadForm {

    if ( $ALLOW_UPLOAD ne 0 ) {
        &Header("File Upload and Message Send");
    }
    else {
        &Header("Message Send");
    }
    if ($PRESEND_FILE ne '') {
        open (PRESEND, "$DATA_ROOT/$PRESEND_FILE") || die "Can't open PRESEND_FILE file. [UploadForm]";
        @list = <PRESEND>;
        close (PRESEND);
        print "@list\n";
    }
    print "<P ALIGN='CENTER'>\n";
    print "<FORM METHOD=POST  ENCTYPE=multipart/form-data >\n";
    print "<table border='0' cellpadding='3' cellspacing='3' width='90%'>\n";
    if ( $ALLOW_UPLOAD ne 0 ) {
        print "<tr><td>File Name:</td>\n";
        print "<td><INPUT TYPE=file NAME=filename SIZE=35></td></tr>\n";
    }
    else {
        print "<INPUT TYPE=hidden NAME=filename>\n";
    }
    print "<tr><td>Subject:</td>\n";
    print "<td><INPUT TYPE=text NAME=subject SIZE=40> \n";
    print "<INPUT TYPE=checkbox NAME=mailself> CC self in mailing\n";
    if ( $ALLOW_UPLOAD ne 0 ) {
        print "<BR><I><FONT SIZE='-1'>Optional if sending attachment, default will be filename.</FONT></I>\n";
    }
    print "</td></tr>\n";
    print "<tr><td>Message Text:</td>\n";
    print "<td><textarea name=comment cols=70 rows=4 wrap=soft></textarea></td></tr>\n";
    print "<tr><td colspan='3' align='center'>\n";
    print "<BR><BR><INPUT TYPE=submit NAME=send VALUE='Send Message'> \n";
    print "<INPUT TYPE=submit NAME=menu VALUE='Exit Message Upload'></td></tr>\n";
    print "</table>\n";
    &WriteUserInfo;
    &Footer;
    exit;
}

#######################################################
# Subroutine to verify and confirm new list members
#######################################################
sub Verify {

    my (@list, $line);

    if ( $VERIFY_FILE ne '' ) {
        @list = &ReadFile( "$DATA_ROOT/$VERIFY_FILE" );
        if ( !@list ) { &Subscribe; }
        &Header("");
        foreach $line ( @list) {
            print "$line\n";
        }
    } else {
      &Header("");
      print "Please confirm your subscription to the list...";
    }
    print "<BR><BR>\n";
    print "<CENTER><FORM METHOD=POST>";
    print "<INPUT TYPE=submit NAME=subscribe VALUE='Confirm'>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Exit</a> ]</p>\n";
    print "<INPUT TYPE=hidden NAME=verify VALUE=1>\n";
    print "<INPUT TYPE=hidden NAME=email VALUE='$email'>\n";
    print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'>\n";
    print "<INPUT TYPE=hidden NAME=password VALUE='$pass'></FORM></CENTER>\n";
    &Footer;
    exit;
}

# end of script
