#!/usr/local/bin/perl 
#-------------------------------------------------------------------
# Only change the location of perl on your server on the line above.
# No other settings should be changed.
#-------------------------------------------------------------------

#############################################################################
# ListMail LIBRARY MODULE
#
# This file contains routines common to all scripts.  Refer to other
# scripts for details on setup and usage.
#############################################################################
# COPYRIGHT NOTICE          Copyright 1998 P.M.Systems  All Rights Reserved.
#
# ListMail may be used and modified free of charge by anyone so long as
# this copyright notice and the comments above remain intact.  By using this
# code you agree to indemnify Paul Mele and P.M.Systems from any liability
# that might arise from it's use.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  In other words, please ask first before you try and
# make money off of my program.
#
# Obtain permission before redistributing this software over the Internet or
# in any other medium.  In all cases copyright and header must remain intact.
#
# Archived at: http://www.pmsys.com/software/    contact: webmaster@pmsys.com
##############################################################################

#-----------------------------------------------------
# security checks
#-----------------------------------------------------

#######################################################
# Displays access denied to user
#######################################################
sub AccessDenied {

    &Header("Access Denied");
    print "<p>You entered an incorrect password and/or user email address \- \n";
    print "Access has been denied!</p>\n";
    print "<center><FORM METHOD=POST>";
    print "<INPUT TYPE=submit NAME=login VALUE=Retry>\n";
    print "<p align='center'>[ <a href='$RETURN_URL'>Cancel</a> ]</p>\n";
    print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'>\n";
    print "</FORM></center>\n"; 
    &Footer;
    exit;
}

#######################################################
# Subroutine to restrict access from specific IP
# addresses.  Useful if someone is abusing service
# from the CGI script. IP list maintained thru LMADM.
#######################################################
sub CheckIp {

    my ($remote_address) = ($ENV{"REMOTE_ADDR"});
    my ($remote_host) = ($ENV{"REMOTE_HOST"});
    my ($ip, $failed, $index, $address_failed, $host_failed, $part);
    my (@address_parts, @host_parts, @ipaddr, @parts);

    open(FILE, "$DATA_ROOT/$IP_FILE") || return;
    @ipaddr = <FILE>;
    close (FILE);
    foreach $ip (@ipaddr) {

        $address_failed = 0;
        $host_failed = 0;

        @parts = split (/./, $ip);
        @address_parts = split (/./, $remote_address);
        @host_parts = split (/./, $remote_host);

        $index = 0;
        foreach $part (@parts) {
            if ($part eq $address_parts[$index] || $part eq "*") { $address_failed++; }
            if ($part eq $host_parts[$index] || $part eq "*") { $host_failed++; }
            $index++;
        }
        if ($address_failed == @parts || $host_failed == @parts) {
            &Error ("IP Address ($ENV{'REMOTE_ADDR'} $ENV{'REMOTE_HOST'}) Not Allowed");
        }
    }
}

#######################################################
# Subroutine to check for both user and password.
# If either is missing, the user is notified and
# denied access to the system.
#######################################################
sub CheckParms {

    my($user, $pass) = @_;

    if ( $user eq '' || $pass eq '' ) {
       &Header("User/Password Missing");
       print "<p>The requested user or password was not entered. Please try again!</p>\n";
       print "<center><FORM METHOD=POST>";
       print "<INPUT TYPE=submit NAME=login VALUE=Retry>\n";
       print "<p align='center'>[ <a href='$RETURN_URL'>Cancel</a> ]</p>\n";
       if ( $GLOBAL{'cfg'} ) {
           print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'>\n";
       }
       print "</FORM></center>\n"; 
       &Footer;
       exit;
    }
}

#######################################################
# Subroutine used to restrict access to allow only a
# specific domain or document to use this script.
# Useful if someone is abusing service from the CGI 
# script. List maintained thru LMADM.
#######################################################
sub CheckReferer {

    my($referer) = ($ENV{"HTTP_REFERER"});
    my ($url, $passed);

    foreach $url (@AUTHORIZED_URLS) {
        if ($referer =~ /^$url/i) { $passed=1; }
    }
    if (!$passed) { &Error ("HTTP Referer $ENV{'HTTP_REFERER'} Not Allowed"); }
}

#######################################################
# Subroutine to restrict access from specific URLS.
# Useful if someone is abusing service from the CGI 
# script. URL list maintained thru LMADM.
#######################################################
sub CheckUrl {

    my($referer) = ($ENV{"HTTP_REFERER"});
    my ($url, @urladdr);

    @urladdr = ReadFile("$DATA_ROOT/$URL_FILE");
    if ( !@urladdr ) { return; }
    foreach $url (@urladdr) {
        if ($referer =~ /^$url/i) { &Error ("HTTP Referer $ENV{'HTTP_REFERER'} Not Allowed"); }
    }
}

#######################################################
# Subroutine to restrict access from specific users.
# Useful if someone is abusing service from a specifc
# email address. Email list maintained thru LMADM.
#######################################################
sub CheckEmail {

    my ($name, @addr);

    @addr = ReadFile("$DATA_ROOT/$EMAIL_FILE");
    if ( !@addr ) { return; }
    foreach $name (@addr) {
        if (lc($name) eq lc($email)) { &Error ("User $email Not Allowed"); }
    }
}

#######################################################
# Subroutine to encrypt password
#######################################################
sub CryptPass {

    my ($password) = @_;
    my (@salts, $salt, $part1, $part2);

    @salts = split (//, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcedfghijklmnopqrstuvwxyz0123456789./");
    srand ();
    $part1 = int (rand($#salts));
    $part2 = int (rand($#salts));
    $salt = "$salts[$part1]$salts[$part2]";
    $password = crypt($password,$salt);
    return ($password);
}

#######################################################
# Subroutine to mail security errors to WebMaster.
# Errors from illegal IP, URL or Email Addresses
# use this routine to send administrative alert and
# to disallow the user access to the system.
#######################################################
sub Error {

    my ($data) = @_;

    # Mail alert to administrator
    open (MAIL, "|$MAILPROG -t") || die "Can't open $MAILPROG";
    print MAIL "To: $FROM\n";
    print MAIL "Subject: *** SECURITY VIOLATION ***\n\n";
    print MAIL "Date: $date\n\n";
    print MAIL "$data\n";
    while (@env_list = each(%ENV)) {
         print MAIL "@env_list\n";
    }
    print MAIL "\n";
    close (MAIL);

    # Disallow access to system
    &Header("Security Error");
    print "<p>Access is not allowed due to \n";
    print "the following security concern:<br><br>$data.</p>\n";
    &Footer;
    exit;
}

#######################################################
# Subroutine to check authorization to mail to list
#######################################################
sub VerifyUserPass {

    my($admin) = @_;
    my($match, $user_pass, $user_name, $salt, $tmp, $file);

    # Which password file are we to use
    if ( $admin ) {
        $file="$DATA_ROOT/$ADMIN_FILE";
    }
    else{ 
        $file="$DATA_ROOT/$USER_FILE";
    }

    # Open password file and compare
    $match = 0;
    open (USER, "$file") || &AccessDenied;
    while (<USER>) {
        chop ($_);
        ($user_pass, $user_name) = split(/\|/, $_);
        if (lc($user_name) eq lc($email)) {
            if ( $encrypt ) {
                # Password not encrypted
                if ($pass eq $user_pass) { $match = 1; }
            }
            else {
                # Password encrypted
                $salt = substr ($user_pass,0,2);
                $tmp = crypt($pass,$salt);
                if ($tmp eq $user_pass) {
                    $pass = $tmp;
                    $match = 1;
                }
            }
        }
    }
    close (USER);

    # If we did not match then deny access to system
    if ( !$match ) { &AccessDenied; }
} 

#-----------------------------------------------------
# Parsing input
#-----------------------------------------------------

#######################################################
# Subroutine to read in vars and parse
#######################################################
sub ParseInput {

    my($i, $loc, $key, $val, $input, @list, @input_list);
    my($f,$header, $header_body, $len, $buf, $cnt, $tmp);

    if ( defined $ENV{'REQUEST_METHOD'} && $ENV{'REQUEST_METHOD'} eq "GET" ) {

         $input = $ENV{'QUERY_STRING'};
    }
    elsif ( defined $ENV{'REQUEST_METHOD'} && $ENV{'REQUEST_METHOD'} eq "POST" ) {

         # Need to read TILL we got all bytes
         $len = 0;
         $input = '';
         while( $len != $ENV{'CONTENT_LENGTH'} ) {
             $buf = '';
             $len += sysread(STDIN, $buf, $ENV{'CONTENT_LENGTH'});
             $input .= $buf;
         }
    }
    $cnt = 0;

    # conform to RFC1867 for upload specific
    if( $ENV{'CONTENT_TYPE'} =~ /multipart\/form-data; boundary=(.+)$/ ) {
        $boundary = '--'.$1;  # please refer to RFC1867
        @list = split(/$boundary/, $input);

        $header_body = $list[1];
        $header_body =~ /\r\n\r\n|\n\n/; # separate header and body
        $header = $`;        # front part
        $body   = $';        # rear part
        $body =~ s/\r\n$//;  # the last \r\n was put in by Netscape
        $GLOBAL{'FILE_CONTENT'} = $body;

        # parse header
        $header =~ /filename=\"(.+)\"/;
        $GLOBAL{'FILE_NAME'} = $1;
        $GLOBAL{'FILE_NAME'} =~ s/\"//g; # remove "s
        $GLOBAL{'FILE_NAME'} =~ s/\s//g; # make sure no space(include \n, \r..) in the file name
 
        # parse trailer
        for( $i=2; $list[$i]; $i++) {
            $list[$i] =~ s/^.+name=$//;
            $list[$i] =~ /\"(\w+)\"/;
            $tmp = $1;
            $GLOBAL{$tmp} = $';
            $GLOBAL{$tmp} =~ s/^\s+//;
            $GLOBAL{$tmp} =~ s/\s+$//;
            $cnt++;
        }
    }
    else {


        @input_list = split(/&/,$input);
        foreach $i (@input_list) {
            ($key, $val) = split(/=/, $i);
            $val =~ tr/+/ /;
            $val =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
            $GLOBAL{$key} = $val;
            $cnt++;
        }
    }
    return ($cnt);
}

#-----------------------------------------------------
# Sending mail messages
#-----------------------------------------------------

#######################################################
# Subroutine to send to all list members
#######################################################
sub DoMailList {

    my($name, $file, $self, $subject, $comment) = @_;
    my(@list, $recip, $sent, $cnt);

    # Actual subscribed members
    @list = &ReadFile( "$DATA_ROOT/$USER_FILE" );
    $sent = 0;
    foreach $line (@list) {

        # split password and user from file
        ($pwd, $recip) = split(/\|/, $line);
        
        MailMsg($recip, $name, $file, $self, $subject, $comment);
        #sleep 2;
        $sent++;
    }

    # Admin receive only addresses.
    @list = &ReadFile( "$DATA_ROOT/$ADDR_FILE" );
    if ( @list ) {
        foreach $line (@list) {
            
            MailMsg($recip, $name, $file, $self, $subject, $comment);
            #sleep 2;
            $sent++;
        }
    }

    # Log message to file
    if ( $LOG_MAIL ne '' ) {
        open (LOG, ">>$DATA_ROOT/$LOG_MAIL") || die "Can't open $LOG_MAIL for output.  [DoMailList]";
        print LOG "Date    = $date\n";
        print LOG "User    = $email\n";
        print LOG "Subject = $subject\n";
        if ( $file ne '' ) { print LOG "Attach = $file\n"; }
        print LOG "Recipients = $sent\n";
        print LOG "=================\n";
        close (LOG);
    }

    # Remove file that was uploaded
    unlink("$file");

    # Return recipient count
    return ($sent);
}

#######################################################
# Subroutine to send email and attachment to list
# Thanks to Michael Budash for this section.
#######################################################
sub MailMsg {

    my($recip, $name, $file, $self, $subject, $comment) = @_;
    my(%table, @file, $b1, $b2, $b3, $b4, $count, $read);

    # Check for blank lines
    # Do not want to send message to user sending message
    if (length($recip) eq 0) { next; }
    if ( lc($recip) eq lc($email) ) {
        if ($self ne 'on') { next; }
    }

    # start message pipe
    open (MAIL, "| $MAILPROG -t -oi -n") || die "Can't open $MAILPROG [MailMsg]";

    # define message type as multipart:
    print MAIL "To: $recip\n";
    print MAIL "From: $FROM\n";
    if ( $subject ne '' ) {
        print MAIL "Subject: $SUBJ_PREFIX $subject\n";
    }
    else {
        print MAIL "Subject: $SUBJ_PREFIX $name\n";
    }
    print MAIL "MIME-Version: 1.0\n";
    print MAIL "Content-Type: multipart/mixed; boundary=\"------------26A28E87DBB\"\n\n";

    # send text part
    print MAIL "--------------26A28E87DBB\n";
    print MAIL "Content-Type: text/plain\n\n";
    if ( length($comment) ne 0 ) {
        print MAIL "============= Sender's Comment =========\n";
        print MAIL "$comment\n\n";
        print MAIL "========================================\n";
    }
    if ( $MSG_FILE ne '' ) {
        open (FILE, "$DATA_ROOT/$MSG_FILE") || die "Can't open $MSG_FILE for input [MailMsg]";
        @file = <FILE>;
        close (FILE);
        print MAIL "@file\n";
    }
    else {
        print MAIL "If you wish to unsubscribe please visit http://$ENV{HTTP_HOST}$ENTRY_URL, ";
        print MAIL "enter your eMail address and password if one was assigned then select Unsubscribe.  ";
        print MAIL "For questions and/or comments, please reply to $FROM.\n";
    }
    print MAIL "\n";

    if ( $file ne '' ) {

        # set up associative array for base64 encoding:
        %table = ('000000', 'A',
          '000001', 'B',
          '000010', 'C',
          '000011', 'D',
          '000100', 'E',
          '000101', 'F',
          '000110', 'G',
          '000111', 'H',
          '001000', 'I',
          '001001', 'J',
          '001010', 'K',
          '001011', 'L',
          '001100', 'M',
          '001101', 'N',
          '001110', 'O',
          '001111', 'P',
          '010000', 'Q',
          '010001', 'R',
          '010010', 'S',
          '010011', 'T',
          '010100', 'U',
          '010101', 'V',
          '010110', 'W',
          '010111', 'X',
          '011000', 'Y',
          '011001', 'Z',
          '011010', 'a',
          '011011', 'b',
          '011100', 'c',
          '011101', 'd',
          '011110', 'e',
          '011111', 'f',
          '100000', 'g',
          '100001', 'h',
          '100010', 'i',
          '100011', 'j',
          '100100', 'k',
          '100101', 'l',
          '100110', 'm',
          '100111', 'n',
          '101000', 'o',
          '101001', 'p',
          '101010', 'q',
          '101011', 'r',
          '101100', 's',
          '101101', 't',
          '101110', 'u',
          '101111', 'v',
          '110000', 'w',
          '110001', 'x',
          '110010', 'y',
          '110011', 'z',
          '110100', '0',
          '110101', '1',
          '110110', '2',
          '110111', '3',
          '111000', '4',
          '111001', '5',
          '111010', '6',
          '111011', '7',
          '111100', '8',
          '111101', '9',
          '111110', '+',
          '111111', '/');

        # send attachment
        print MAIL "--------------26A28E87DBB\n";
        print MAIL "Content-Type: application/octet-stream\n";
        print MAIL "Content-Transfer-Encoding: base64\n";
        print MAIL "Content-Disposition: attachment; filename=\"$name\"\n\n";

        # print file to attach
        open (FILE, "$file") || &UploadFailed("$file  $!");

        $count = 1;
        while(($read = read(FILE, $buf, 3))) {
            $u = unpack("B24", $buf);
            if ($read eq 1) { $u .= "0000000000000000"; }
            if ($read eq 2) { $u .= "00000000"; }

            $b1 = $table{substr($u, 0, 6)};
            $b2 = $table{substr($u, 6, 6)};
            $b3 = $table{substr($u, 12, 6)};
            $b4 = $table{substr($u, 18, 6)};

            if ($read eq 3) { print MAIL "$b1$b2$b3$b4"; }
            if ($read eq 2) { print MAIL "$b1$b2$b3="; }
            if ($read eq 1) { print MAIL "$b1$b2=="; }

            if (++$count eq 19) {
                print MAIL "\n";
                $count = 1;
            }
        }
        close (FILE);
    }

    # close message and pipe
    print MAIL "\n--------------26A28E87DBB--";
    close MAIL;
}

#-----------------------------------------------------
# File locking, unlocking and misc
#-----------------------------------------------------

#######################################################
# Subroutine to check/lock file 
#######################################################
sub CheckLock {

    my($time) = @_;

    for ($i = 1;$i <= $time; $i++) {
        if (-e "$DATA_ROOT/$LOCK_FILE") {
            sleep( 1 );
        }
        else {
            open(LOCK,">$DATA_ROOT/$LOCK_FILE");
            print LOCK "0";
            close(LOCK);
            last;
        }
    }
}
 
#######################################################
# Unlock file
#######################################################
sub CleanUp {

    unlink("$DATA_ROOT/$LOCK_FILE");
}

#######################################################
## Read from file into an array
#######################################################
sub ReadFile {

    my($name) = @_;
    my(@file, $tmp, $cnt, @rtn);

    # Input file
    open(FILE, "$name") || return ();
    @file = <FILE>;
    close (FILE);

    # Put addresses into an array
    $cnt = 0;
    $tmp = join('',@file);
    @file = split(/\n/,$tmp);
    foreach $tmp (@file) {
        $tmp =~ s/\s*(.*?)\s*$/$1/;
        if ( $tmp ne '' ) {
            @rtn[$cnt] = $tmp;
            $cnt++;
        }
    }
    return ( @rtn );
}

#######################################################
# Subroutine to display upload failed to user
#######################################################
sub UploadFailed {

    my($reason) = @_;

    &Header("Upload Failed");
    print "<p>The requested object was not uploaded to the server.\n";
    print "<br><br>Reason : $reason.<br><br>";
    print "The server may have decided not let you \n";
    print "write to the directory specified. Please contact the webmaster \n";
    print "for this problem. Connection closed by foreign host.</p>\n";
    print "<center>\n"; 
    print "<FORM METHOD=POST>";
    print "<INPUT TYPE=submit NAME=menu VALUE=' Menu '>\n";
    &WriteUserInfo;
    print "</center>\n"; 
    &Footer;
    exit;
}

#######################################################
# Subroutine to upload file to server and save to disk
#######################################################
sub UploadFile {

    my($local_file, $remote_file) = @_;

    if( !$local_file ) {
        &Header("Object Not Found");
        print "<p>The requested object '$local_file' could not be found! ";
        print "The link you followed is either outdated, inaccurate, ";
        print "or the server has been instructed not to let you have it. ";
        print "Connection closed by foreign host.</p>\n";
        print "<center>\n"; 
        print "<FORM METHOD=POST>";
        print "<INPUT TYPE=submit NAME=menu VALUE=' Menu '>\n";
        &WriteUserInfo;
        print "</center>\n"; 
        &Footer;
        exit;
    }
    open(ULFD,">$remote_file")  || &UploadFailed("$remote_file  $!");
    print ULFD $GLOBAL{'FILE_CONTENT'};
    close(ULFD);
}

#-----------------------------------------------------
# HTML page
#-----------------------------------------------------

#######################################################
# Subroutine to display common html footer
#######################################################
sub Footer {

    print "<HR><BR>\n";
    print "<CENTER><FONT SIZE=-1><I>\n";
    print "Comments, questions or problems? \n";
    print "Mail to <a HREF='mailto:$FROM'>$FROM</a>\n";
    print "</CENTER><p>ListMail Version $VN  &nbsp;&nbsp;&copy;<a href='http://www.pmsys.com/software/'>P.M.Systems</a></I></FONT>\n";
    print "</body>\n";
    print "</html>\n";
}

#######################################################
# Subroutine to display common html header and context
#######################################################
sub Header {

    my ($title) = @_;

    print "Content-type: text/html\n\n";
    print "<html>\n";
    print "<head>\n";
    print "<title>$title</title>\n";
    print "</head>\n";
    print "<body";
    if ($BGCOLOR ne "") { print " bgcolor='$BGCOLOR'"; }
    if ($TEXTCOLOR ne "") { print " text='$TEXTCOLOR'"; }
    if ($LINK_COLOR ne "") { print " link='$LINK_COLOR'"; }
    if ($ALINK_COLOR ne "") { print " link='$ALINK_COLOR'"; }
    if ($VLINK_COLOR ne "") { print " link='$VLINK_COLOR'"; }
    if ($BGIMAGE ne "") { print " background='$BGIMAGE' bgproperties='fixed'"; }
    print ">\n";
    print "<h1>$title</h1>\n";
}

#######################################################
# Subroutine to write user and password info to html form
#######################################################
sub WriteUserInfo {
    print "<INPUT TYPE=hidden NAME=cfg VALUE='$cfgdata'>\n";
    print "<INPUT TYPE=hidden NAME=encrypt VALUE=1>\n";
    print "<INPUT TYPE=hidden NAME=email VALUE='$email'>\n";
    print "<INPUT TYPE=hidden NAME=password VALUE='$pass'></FORM>\n";
}

# end of script
1; #return true 
